//-------------------------------------------------------------------
//	File:		YourCode.cpp
//	Comments:	Write your visual effect here.
//-------------------------------------------------------------------
#include "stdafx.h"
#include "Texture.h"
#include "CObject.h"

int NBOBJ=4;
#define NBOBJMAX 10
#define USELIST 1
#define USESTRIP
#define OBJSIZE 2000

//#define DATAREP "h:\\data\\"
#define DATAREP 

//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Hufo's dancing objects";

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;
extern HWND g_hwnd;
extern bool bFullScreen;
// Use the demonstration texture.
//CTexture gTexExample;

CObject obj;

bool preInit()
{
	Test(obj.LoadLWO(DATAREP "fig22.lwo"));
#ifdef USESTRIP
	Test(obj.GetStrip(DATAREP "fig22_Default.objf"));
#endif
	switch(MessageBox(NULL,
		"Keys:\n"
		" 1...9: set nb objects\n"
		" SPACE: enable/disable transparency\n"
		" ENTER: enable/disable strip optimisation\n"
		"\n"
		" DO YOU WANT TO GO FULLSCREEN (same res/colors as desktop)\n"
		"Note: fullscreen mode may crash your computer. use it at your own risk\n"
		,szAppName
		,MB_YESNOCANCEL|MB_ICONQUESTION|MB_DEFBUTTON2
		))
	{
		case IDYES:
			bFullScreen=true;
			break;
		case IDNO:
			bFullScreen=false;
			break;
		case IDCANCEL:
			return r_err;
	}
	return r_ok;
}

float ObjTr[NBOBJMAX][3]/*=
{{1000.0f,0.0f,0.0f}
,{0.0f,1000.0f,0.0f}
,{-1000.0f,0.0f,0.0f}
,{0.0f,-1000.0f,0.0f}
}*/;

void SetNbObj(int n)
{
	NBOBJ=min(NBOBJMAX,n);
	if (NBOBJ==1)
	{
		ObjTr[0][0]=0.0f;
		ObjTr[0][1]=0.0f;
		ObjTr[0][2]=0.0f;
	}
	else
	for (int i=0;i<NBOBJ;++i)
	{
		ObjTr[i][0]=(float)((OBJSIZE/2)*cos(i*2*PI/NBOBJ));
		ObjTr[i][1]=(float)((OBJSIZE/2)*sin(i*2*PI/NBOBJ));
		ObjTr[i][2]=0.0f;
	}
}

#ifdef USELIST
int ObjList;
bool ObjChanged;
#endif
#ifdef USESTRIP
bool ObjStrip;
#endif
int ClearBuf;
// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	SetNbObj(NBOBJ);
	// Reset the matrix to something we know.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float x = (float)g_w/(float)g_h; // Correct the viewing ratio of the window in the X axis.
	gluOrtho2D( -x, x, -1, 1 );	// Reset to a 2D screen space.

	// Reset model view matrix stack.
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

//	gTexExample.LoadJPG( "Demo.jpg" );
//	glEnable( GL_TEXTURE_2D );
//	gTexExample.Use(); // Tells OpenGL to use this texture.

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( 110, x, 100.0f, 2*OBJSIZE); //mfFarZ );
//	gluLookAt(	vPos.x, vPos.y, vPos.z,
//				vDir.x, vDir.y, vDir.z,
//				vUp.x, vUp.y, vUp.z);

	// Reset model view matrix stack.
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	//glCullFace(GL_FRONT); // reject fliped faces

	glDepthFunc(GL_LESS);
	if (obj.sf[0].flag&CMaterial::Blend)
	{
		glDisable(GL_DEPTH_TEST);
		ClearBuf=GL_COLOR_BUFFER_BIT;
	}
	else
	{
		glEnable(GL_DEPTH_TEST);
		ClearBuf=GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;
	}

	//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE);

	//Test(obj.LoadLWO("h:\\data\\fig22.lwo"));
#ifdef USELIST
	ObjChanged=true; // rebuild list
	ObjList=glGenLists(1);
	if (!ObjList) return HErrorN("glList creation error");
#endif
#ifdef USESTRIP
	ObjStrip=true;
#endif
	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return r_ok;
}

void Shutdown() // Called right after the window is destroyed.
{
#ifdef USELIST
	if (ObjList)
	{
		glDeleteLists(ObjList,1);
		ObjList=0;
	}
#endif
	obj.Free();
}

void UserKey(int k)
{
	switch (k)
	{
#ifdef USESTRIP
		case VK_RETURN:
			ObjStrip=!ObjStrip;
#ifdef USELIST
			ObjChanged=True; // rebuild list
#endif
			break;
#endif
		case VK_SPACE:
			if (obj.sf[0].flag&CMaterial::Blend)
			{
				obj.sf[0].flag&=~(CMaterial::Blend|CMaterial::Additive|CMaterial::Double);
				glEnable(GL_DEPTH_TEST);
				ClearBuf=GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;
			}
			else
			{
				obj.sf[0].flag|=(CMaterial::Blend|CMaterial::Additive|CMaterial::Double);
				glDisable(GL_DEPTH_TEST);
				ClearBuf=GL_COLOR_BUFFER_BIT;
			}
#ifdef USELIST
			ObjChanged=True; // rebuild list
#endif
			break;
		case VK_ESCAPE:
			PostMessage(g_hwnd,WM_CLOSE,0,0);
			break;
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			SetNbObj(k-'0');
			break;
	}
}

// Draw all the scene related stuff.
void Render()
{
	static float secTime=fTime;
	static int nf=0;
	if (fTime-secTime>=1.0)
	{ // aff framerate
		char buf[256];
		sprintf(buf,"%s - %g fps (%d polys)",szAppName,nf/(fTime-secTime),NBOBJ*obj.nbf);
		SetWindowText(g_hwnd,buf);
		secTime=fTime;
		nf=0;
	}
	++nf;

	glClear( ClearBuf );
	glPushMatrix();
		float light0[4]={0.0f,0.0f,1.0f,0.0f};
		glEnable(GL_NORMALIZE);
		if (NBOBJ==1)
			glTranslatef(0.0f,0.0f,-OBJSIZE/2);
		else
			glTranslatef(0.0f,0.0f,-OBJSIZE);
		// Calculate light position
		glPushMatrix();
			glRotatef( 180+fTime*180, 0.0f, 1.0f, 0.0f );
			glRotatef( fTime*100, 1.0f, 0.0f, 0.0f );
			glRotatef( fTime* 70, 0.0f, 0.0f, 1.0f );
			glLightfv(GL_LIGHT0,GL_POSITION,light0);
			glEnable(GL_LIGHT0);
		glPopMatrix();
		// Calculate objects position
		glRotatef( fTime*30, 0.0f, 1.0f, 0.0f );
		glRotatef(90.0f, 1.0f, 0.0f, 0.0f );
		for (int i=0;i<NBOBJ;++i)
		{
			glPushMatrix();
			glTranslatef(ObjTr[i][0],ObjTr[i][1],ObjTr[i][2]);
			glRotatef(-fTime*80, 0.0f, 1.0f, 0.0f );
#ifdef USELIST
			if (ObjChanged) // rebuild list
			{
				glNewList(ObjList,GL_COMPILE_AND_EXECUTE);
#endif
#ifdef USESTRIP
				if (ObjStrip)
					obj.AffStrip();
				else
#endif
				obj.Aff();
#ifdef USELIST
				glEndList();
				ObjChanged=false;
			}
			else
				glCallList(ObjList);
#endif
			glPopMatrix();
		}
	glPopMatrix();
}
